/**
 * ADT-Klasse Menge (nur ein Ausschnitt)
 * 
 */
class Menge {

	private int[] werte;
	private int next;

	/**
	 * Konstruktor; initialisiert leer Menge
	 * 
	 * @param maxSize
	 *            maximal Groesse der Menge (muss groesser als 0 sein!)
	 */
	public Menge(int maxSize) {
		this.werte = new int[maxSize];
		this.next = 0;
	}

	/**
	 * Copy-Konstruktor
	 * 
	 * @param menge
	 *            zu klonierende Menge
	 */
	public Menge(Menge menge) {
		this(menge.werte.length);
		this.next = menge.next;
		for (int i = 0; i < this.next; i++) {
			this.werte[i] = menge.werte[i]; // elementweises Kopieren!
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	protected Menge clone() {
		return new Menge(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (this.getClass() != obj.getClass()) {
			return false;
		}
		Menge menge = (Menge) obj;
		if (this.next != menge.next) {
			return false;
		}
		for (int i = 0; i < this.next; i++) {
			boolean enthalten = false;
			for (int j = 0; j < this.next; j++) {
				if (menge.werte[j] == this.werte[i]) {
					enthalten = true;
				}
			}
			if (!enthalten) {
				return false;
			}
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String str = "[";
		for (int i = 0; i < this.next - 1; i++) {
			str += this.werte[i] + ", ";
		}
		if (this.next - 1 >= 0) {
			str += this.werte[this.next - 1];
		}
		str += "]";
		return str;
	}

	/**
	 * ueberprueft, ob die Menge leer ist
	 * 
	 * @return genau dann true, wenn Menge leer ist
	 */
	public boolean istLeer() {
		return this.next == 0;
	}

	/**
	 * ueberprueft, ob die Menge voll ist
	 * 
	 * @return genau dann true, wenn Menge voll ist
	 */
	public boolean istVoll() {
		return this.next == this.werte.length;
	}

	/**
	 * ueberprueft, ob ein Wert in der Menge enthalten ist
	 * 
	 * @param elem
	 *            zu ueberpruefender Wert
	 * @return genau dann true, wenn elem in der Menge enthalten ist
	 */
	public boolean istEnthalten(int elem) {
		for (int i = 0; i < this.next; i++) {
			if (this.werte[i] == elem) {
				return true;
			}
		}
		return false;
	}

	/**
	 * fuegt int-Wert in Menge hinzu
	 * 
	 * @param elem
	 *            hinzuzufuegender Wert
	 * @return genau dann true, wenn der int-Wert zur Menge hinzugefuegt wurde
	 */
	public boolean hinzufuegen(int elem) {
		if (!this.istVoll() && !this.istEnthalten(elem)) {
			this.werte[this.next] = elem;
			this.next++;
			return true;
		} else {
			return false;
		}
	}

	/**
	 * berechnet Schnittmenge zweier Mengen
	 * 
	 * @param m1
	 *            Menge 1 (muss != null sein)
	 * @param m2
	 *            Menge 2 (muss != null sein)
	 * @return Schnittmenge von m1 und m2
	 */
	public static Menge schnittmenge(Menge m1, Menge m2) {
		Menge menge = new Menge(Math.min(m1.werte.length, m2.werte.length));
		for (int i = 0; i < m1.next; i++) {
			for (int j = 0; j < m2.next; j++) {
				if (m1.werte[i] == m2.werte[j]) {
					menge.hinzufuegen(m1.werte[i]);
				}
			}
		}
		return menge;
	}

	/**
	 * berechnet Vereinigung zweier Mengen
	 * 
	 * @param m1
	 *            Menge 1 (muss != null sein)
	 * @param m2
	 *            Menge 2 (muss != null sein)
	 * @return Vereinigung von m1 und m2
	 */
	public static Menge vereinigung(Menge m1, Menge m2) {
		Menge menge = new Menge(m1);
		for (int i = 0; i < m2.next; i++) {
			menge.hinzufuegen(m2.werte[i]);
		}
		return menge;
	}
}

public class UE21Aufgabe13 {

	// kleines Testprogramm (nur ein Ausschnitt)
	public static void main(String[] args) {
		Menge m1 = new Menge(10);
		m1.hinzufuegen(2);
		m1.hinzufuegen(3);
		m1.hinzufuegen(2);
		System.out.println(m1);

		Menge m2 = new Menge(5);
		m2.hinzufuegen(1);
		m2.hinzufuegen(3);
		m2.hinzufuegen(5);
		System.out.println(m2);

		Menge m3 = Menge.schnittmenge(m1, m2);
		System.out.println(m3);

		Menge m4 = Menge.vereinigung(m1, m2);
		System.out.println(m4);
	}

}
